package cz.muni.fi.pb162.hw01.impl;

import com.beust.jcommander.Parameter;
import cz.muni.fi.pb162.hw01.CipherType;
import cz.muni.fi.pb162.hw01.Operation;
import cz.muni.fi.pb162.hw01.cmd.CipherTypeConverter;
import cz.muni.fi.pb162.hw01.cmd.CommandLine;
import cz.muni.fi.pb162.hw01.cmd.OperationConverter;
import cz.muni.fi.pb162.hw01.impl.ciphers.CeaserCipher;
import cz.muni.fi.pb162.hw01.impl.ciphers.Cipher;
import cz.muni.fi.pb162.hw01.impl.ciphers.MorseCode;
import cz.muni.fi.pb162.hw01.impl.ciphers.VigenereCipher;

/**
 * Application class represents the command line interface of this application.
 *
 * You are expected to implement  the  {@link Application#run()} method
 *
 * @author jcechace
 */
public class Application {

    @Parameter(names = {"--cipher", "-c"}, converter = CipherTypeConverter.class)
    private CipherType cipherType = CipherType.MORSE_CODE;

    @Parameter(names = {"--operation", "-o"}, required = true, converter = OperationConverter.class)
    private Operation operation;

    @Parameter(names = {"--text", "-t"}, required = true)
    private String text;

    @Parameter(names = {"--shift"})
    private int caesarShift;

    @Parameter(names = {"--key"})
    private String vigenereKey;

    @Parameter(names = "--help", help = true)
    private boolean showUsage = false;

    /**
     * Application entry point
     *
     * @param args command line arguments of the application
     */
    public static void main(String[] args) {
        Application app = new Application();

        CommandLine cli = new CommandLine(app);
        cli.parseArguments(args);

        if (app.showUsage) {
            cli.showUsage();
        } else {
            app.run();
        }
    }

    /**
     * Application runtime logic
     */
    private void run() {
        Cipher cipher = null;
        switch(cipherType){
            case CAESAR:
                cipher = new CeaserCipher(caesarShift);
                break;
            case VIGENERE:
                cipher = new VigenereCipher(vigenereKey);
                break;
            case MORSE_CODE:
                cipher = new MorseCode();
                break;
            default:
                System.err.println("Unknown cipher");
                System.exit(1);
                
        }
        switch (operation){
            case ENCRYPT: 
                System.out.println(cipher.encrypt(text));
                break;
            case DECRYPT:
                System.out.println(cipher.decrypt(text));
                break;
            default:
                System.err.println("Unknown operation");
                System.exit(2);
        }
    }
}
